/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PoolVisitor.java 5512 2010-05-27 15:14:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.visitor;

import org.ow2.util.pool.api.IPoolConfiguration;
import org.ow2.util.pool.api.IPoolMetadata;
import org.ow2.util.pool.impl.PoolConfiguration;
import org.ow2.util.scan.api.IAnnotationVisitor;

/**
 * This class manages the handling of &#64;{@link org.ow2.util.pool.annotation.Pool}
 * annotation.
 * @author Florent Benoit
 */
public class PoolVisitor implements IAnnotationVisitor {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Lorg/ow2/util/pool/annotation/Pool;";

    /**
     * Min attribute of the annotation.
     */
    private static final String MIN = "min";

    /**
     * Spare attribute of the annotation.
     */
    private static final String SPARE = "spare";

    /**
     * Max attribute of the annotation.
     */
    private static final String MAX = "max";

    /**
     * Timeout attribute of the annotation.
     */
    private static final String TIMEOUT = "timeout";

    /**
     * MaxWaiters attribute of the annotation.
     */
    private static final String MAX_WAITERS = "maxwaiters";

    /**
     * Metadata to complete if there is the &#64;{@link org.ow2.util.pool.annotation.Pool}
     * annotation.
     */
    private IPoolMetadata poolMetadata = null;

    /**
     * Constructor.
     * @param poolMetadata linked to a metadata allowing settings for a Pool.
     */
    public PoolVisitor(final IPoolMetadata poolMetadata) {
        this.poolMetadata = poolMetadata;
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    public void visitEnd() {
        // Sets the pool configuration even if there wasn't any parameter in
        // this annotation
        poolMetadata.setPoolConfiguration(getPoolConfiguration());
    }

    /**
     * @return the object representing common bean.
     */
    public IPoolConfiguration getPoolConfiguration() {
        IPoolConfiguration poolConfiguration = poolMetadata.getPoolConfiguration();
        if (poolConfiguration == null) {
            poolConfiguration = new PoolConfiguration();
            poolMetadata.setPoolConfiguration(poolConfiguration);
        }
        return poolConfiguration;
    }

    /**
     * Visits a primitive value of the annotation.
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    public void visit(final String name, final Object value) {
        if (MAX.equals(name)) {
            getPoolConfiguration().setMax(((Integer) value).intValue());
        } else if (MIN.equals(name)) {
            getPoolConfiguration().setMin(((Integer) value).intValue());
        } else if (SPARE.equals(name)) {
            getPoolConfiguration().setSpare(((Integer) value).intValue());
        } else if (TIMEOUT.equals(name)) {
            getPoolConfiguration().setMaxWaiters(((Integer) value).intValue());
        } else if (MAX_WAITERS.equals(name)) {
            getPoolConfiguration().setTimeout(((Long) value).longValue());
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    public void visitEnum(final String name, final String desc, final String value) {

    }

    /**
     * Visits a nested annotation value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the nested annotation class.
     * @return a non null visitor to visit the actual nested annotation value.
     *         <i>The nested annotation value must be fully visited before
     *         calling other methods on this annotation visitor</i>.
     */
    public IAnnotationVisitor visitAnnotation(final String name, final String desc) {
        return this;
    }

    /**
     * Visits an array value of the annotation.
     * @param name the value name.
     * @return a non null visitor to visit the actual array value elements. The
     *         'name' parameters passed to the methods of this visitor are
     *         ignored. <i>All the array values must be visited before calling
     *         other methods on this annotation visitor</i>.
     */
    public IAnnotationVisitor visitArray(final String name) {
        return this;
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

}
